package com.atlassian.confluence.jspwiki;

import com.atlassian.confluence.importexport.DefaultExportContext;
import com.atlassian.confluence.jspwiki.util.MemorySettingsManager;
import com.atlassian.confluence.setup.BootstrapManager;
import com.atlassian.confluence.setup.BootstrapUtils;
import com.atlassian.confluence.setup.ConfluenceApplicationConfig;
import com.atlassian.confluence.setup.DefaultBootstrapManager;
import com.ecyrd.jspwiki.PageManager;
import com.ecyrd.jspwiki.attachment.AttachmentManager;
import net.sf.hibernate.SessionFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.Properties;

public class ConfluenceExporter
{
    static AttachmentManager attachmentManager;
    static PageManager pageManager;
    static String spaceName;
    static String creatorUserName;
    static Properties properties = new Properties();

    public static void main(String[] args) throws Exception
    {
        if (args.length < 3 || args.length > 3)
        {
            System.out.println("syntax: java -jar jspwiki-exporter-0.1.jar path/to/jspwiki.properties spaceName creatorUserName");
            return;
        }

        File jspwikiPropertiesFile = null;
        File file = new File(args[0]);
        if (file.isFile())
        {
            jspwikiPropertiesFile = file;
        }
        else if (file.isDirectory())
        {
            jspwikiPropertiesFile = new File(file, "jspwiki.properties");
        }

        spaceName = args[1];
        creatorUserName = args[2];

        if (jspwikiPropertiesFile == null)
            throw new FileNotFoundException("Error accessing jspwiki.properties: Please check the path to this file is correct");

        System.out.print("Exporting ");
        ProgressIndicator progressIndicator = new ProgressIndicator();
        progressIndicator.setPriority(Thread.MIN_PRIORITY);
        progressIndicator.start();
        String pathToExportZip = export(jspwikiPropertiesFile);

        progressIndicator.dieSilently();
        progressIndicator.join();
        System.out.println(" complete.");
        System.out.println("Export file: " + pathToExportZip);
        System.exit(0);
    }

    private static class ProgressIndicator extends Thread
    {
        private boolean finished = false;

        public void run()
        {
            super.run();

            while (!finished)
            {
                System.out.print(".");
                try
                {
                    sleep(2000);
                }
                catch (InterruptedException e)
                {

                }
            }
        }

        public void dieSilently()
        {
            finished = true;
            interrupt();
        }
    }

    public static String export(File jspwikiPropertiesFile) throws Exception
    {
        instantiateManagers(jspwikiPropertiesFile);
        return exportEverything();
    }

    static void instantiateManagers(File jspwikiPropertiesFile) throws Exception
    {
        InputStream propertyStream = null;
        propertyStream = new FileInputStream(jspwikiPropertiesFile);

        properties.load(propertyStream);

        pageManager = new PageManager(properties);
        attachmentManager = new AttachmentManager(null, properties);
    }

    static String exportEverything() throws Exception
    {
        Logger.getRootLogger().setLevel(Level.ERROR);

        // hack (because CustomClobType's contructor needs the following to not be null)
        DefaultBootstrapManager bootstrapManager = new DefaultBootstrapManager();
        bootstrapManager.setApplicationConfig(new ConfluenceApplicationConfig());
        BootstrapUtils.setBootstrapManager(bootstrapManager);

        // the export zip and the temporary working space of jspwiki-exporter occurs inside BootstrapManager.TEMP_DIR_PROP
        // create an "exports" directory inside the current working directory to serve this purpose
        String workingDirectory = System.getProperty("user.dir");
        bootstrapManager.setProperty(BootstrapManager.TEMP_DIR_PROP, workingDirectory + File.separator + "exports");

        JspWikiBackupExporter jspWikiBackupExporter = new JspWikiBackupExporter(pageManager, attachmentManager, spaceName, creatorUserName, properties);
        DefaultExportContext defaultExportContext = new DefaultExportContext();
        defaultExportContext.setExportAttachments(true);
        jspWikiBackupExporter.setContext(defaultExportContext);

        // xmldatabinder of the exporter needs a sessionFactory
        ClassPathXmlApplicationContext jspWikiAppContext = new ClassPathXmlApplicationContext("/jspwikiApplicationContext.xml");
        jspWikiBackupExporter.setSessionFactory((SessionFactory) jspWikiAppContext.getBean("sessionFactory"));

        // feed a bootstrapManager to jspWikiBackupExporter to avoid npe. settings manager needed to retrieve default encoding.
        jspWikiBackupExporter.setBootstrapManager(bootstrapManager);
        jspWikiBackupExporter.setSettingsManager(new MemorySettingsManager());

        return jspWikiBackupExporter.doExport();
    }

}
